from django.utils.six import string_types


def get_ct(model, for_concrete_model=False):
    from .models import ContentType
    if isinstance(model, string_types):
        ctype = ContentType.objects.get_by_identifier(model)
    else:
        ctype = ContentType.objects.get_for_model(
            model, for_concrete_model=for_concrete_model
        )
        # get_for_model sometimes returns a DjangoContentType - caching issue?
        if not isinstance(ctype, ContentType):
            ctype = ContentType.objects.get(pk=ctype.pk)
            ContentType.objects._add_to_cache(ContentType.objects.db, ctype)
    return ctype


def get_object_id(instance):
    ct = get_ct(instance)
    config = ct.get_config()
    if config and 'lookup' in config:
        return getattr(instance, config['lookup'])
    return instance.pk


def get_by_identifier(queryset, ident):
    if hasattr(queryset, 'get_by_identifier'):
        return queryset.get_by_identifier(ident)
    else:
        ct = get_ct(queryset.model)
        config = ct.get_config()
        if config and 'lookup' in config:
            lookup = config['lookup']
        else:
            lookup = 'pk'
        return queryset.get(**{lookup: ident})



from . import router
from django.utils.http import urlquote


def version(request):
    return {'version': router.version}


def router_info(request):
    # FIXME: support non-root base_url
    base_url = ""
    path = request.path[1:]
    if request.GET:
        path += "?" + request.GET.urlencode()
    full_path = base_url + "/" + path

    info = {
        'base_url': base_url,
        'path': path,
        'path_enc': urlquote(path),
        'params': request.GET,
        'full_path': full_path,
        'full_path_enc': urlquote(full_path),
        'prev_path': '',  # Referer?
    }

    return {
        'router_info': info,
        'pages_info': info,  # FIXME: Remove in 2.0
    }


# FIXME: Remove in 2.0
def pages_info(request):
    return router_info(request)


def wq_config(request):
    # FIXME: support non-root base_url
    parts = request.path.split('/')
    user = request.user if request.user.is_authenticated else None
    wq_conf = router.get_config(user=user)
    page_conf = None
    root_conf = None

    for name, conf in wq_conf['pages'].items():
        if parts[1] == conf['url']:
            page_conf = conf
        elif conf['url'] == "":
            root_conf = conf

    if not page_conf and root_conf and len(parts) == 2:
        page_conf = root_conf

    return {
        'wq_config': wq_conf,
        'page_config': page_conf,
    }



from .base.serializers import (
    AttachmentSerializer,
    TypedAttachmentSerializer,
    AttachedModelSerializer,
    NaturalKeyModelSerializer,
    NaturalKeyAttachedModelSerializer,
)

from .identify.serializers import IdentifiedModelSerializer


__all__ = (
    'AttachmentSerializer',
    'TypedAttachmentSerializer',
    'AttachedModelSerializer',
    'NaturalKeyModelSerializer',
    'NaturalKeyAttachedModelSerializer',
    'IdentifiedModelSerializer',
)



from django.core.management.base import BaseCommand
from wq.db import rest
from rest_framework.utils import encoders
import json


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument(
            '--format',
            default='json',
        )

    def handle(self, **options):
        text = json.dumps(
            rest.router.get_config(),
            cls=encoders.JSONEncoder,
            indent=4,
        )
        if options['format'] == "amd":
            text = "define(%s);" % text
        self.stdout.write(text)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-01-12 03:18
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Authority',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('homepage', models.URLField(blank=True, null=True)),
                ('object_url', models.URLField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
                'verbose_name_plural': 'authorities',
                'db_table': 'wq_identifiertype',
            },
        ),
        migrations.CreateModel(
            name='Identifier',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=255)),
                ('slug', models.SlugField(blank=True)),
                ('is_primary', models.BooleanField(default=False)),
                ('object_id', models.PositiveIntegerField()),
                ('authority', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='identify.Authority')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'abstract': False,
                'ordering': ['-is_primary', 'authority', 'name'],
                'db_table': 'wq_identifier',
            },
        ),
    ]



from wq.db.rest.serializers import ModelSerializer


class UserSerializer(ModelSerializer):
    class Meta:
        exclude = ('id', 'password', 'user_permissions', 'groups',)
        read_only_fields = ('last_login', 'date_joined')
